#!/usr/bin/env python

from os import mkdir, makedirs, symlink
from os.path import exists, extsep, join, splitext
from zipfile import ZipFile
from cStringIO import StringIO
import codecs
import xmlread

class ConfluenceHandler:

    "Handle content from a Confluence Wiki dump."

    def __init__(self, directory):
        self.content = {}
        self.elements = []
        self.directory = directory

    def handle_object(self, name, elements, attributes, all_text, text):

        "Handle objects according to type."

        objecttype = attributes[-1]["class"]
        identifier = text.strip()
        content = self.content

        pages_dir = join(self.directory, "pages")
        versions_dir = join(self.directory, "versions")

        # Handle particular types.

        if objecttype == "Page":

            # Handle pages.

            if content.has_key("historicalVersions"):
                versions = content["historicalVersions"]

                # Make a page directory and links from the versions to separate
                # files.

                page_dir = join(pages_dir, identifier)
                revisions_dir = join(page_dir, "revisions")

                mkdir(page_dir)
                mkdir(revisions_dir)

                # Historical versions are stored separately.

                for n, (cls, version) in enumerate(versions):
                    if cls == "Page":
                        symlink(join("..", "..", "..", "..", versions_dir, version), join(revisions_dir, str(n+1)))

                # The page always seems to hold the current version itself.

                symlink(join("..", "..", "..", "..", versions_dir, identifier), join(revisions_dir, str(len(versions)+1)))

            # Handle pages and revisions, adding revisions to the page manifest.

            if content.has_key("originalVersion"):
                pageid = content["originalVersion"]
            else:
                pageid = identifier

            versionfile = join(versions_dir, identifier)

            # Note page metadata, not necessarily in the correct order.

            mkdirs(join(pages_dir, pageid))

            append(join(pages_dir, pageid, "import"), "%s|AddRevision|%s|%s|%s|%s\n" % (
                content["version"],
                versionfile,
                content["title"],
                content["lastModifierName"],
                content["versionComment"]))

            # Some metadata is not particularly relevant. For example,
            # ancestors, children, parent are navigation-related.

            # Other metadata could be added to the page content itself.
            # For example, labelling could be converted to categories.

        # Handle revisions.

        elif objecttype == "BodyContent":
            write(join(versions_dir, content["content"]), content["body"])

        self.content = {}

    def handle_property(self, name, elements, attributes, all_text, text):

        "Record properties in the current content dictionary."

        self.content[attributes[-1]["name"]] = text.strip()

    def handle_id(self, name, elements, attributes, all_text, text):

        "Promote identifiers to the parent element's text."

        all_text[-2].append(text)

    def handle_collection(self, name, elements, attributes, all_text, text):

        "Record collections in the current content dictionary."

        self.content[attributes[-1]["name"]] = self.elements
        self.elements = []

    def handle_element(self, name, elements, attributes, all_text, text):

        "Add elements to the current collection."

        self.elements.append((attributes[-1]["class"], text.strip()))

def mkdirs(name):
    try:
        makedirs(name)
    except OSError:
        pass

def append(filename, s):
    write(filename, s, True)

def write(filename, s, append=False):
    f = codecs.open(filename, append and "ab" or "wb", encoding="utf-8")
    try:
        f.write(s)
    finally:
        f.close()

if __name__ == "__main__":
    import sys

    filename = sys.argv[1]
    is_zipfile = splitext(filename)[-1] == extsep + "zip"

    directory = sys.argv[2]

    if exists(directory):
        print >>sys.stderr, "Directory exists. Please choose another or remove its contents."
        sys.exit(1)

    mkdir(directory)
    mkdirs(join(directory, "pages"))
    mkdirs(join(directory, "versions"))

    p = xmlread.ConfigurableParser()
    handler = ConfluenceHandler(directory)

    p["object"] = handler.handle_object
    p["property"] = handler.handle_property
    p["id"] = handler.handle_id
    p["collection"] = handler.handle_collection
    p["element"] = handler.handle_element

    f = open(filename)

    if is_zipfile:
        zf = ZipFile(f)
        ff = StringIO(zf.read("entities.xml"))
    else:
        ff = f

    try:
        p.parse(ff)
    finally:
        f.close()

# vim: tabstop=4 expandtab shiftwidth=4
